import React, { useContext, useState, useEffect, useRef } from "react"
import {
	Table,
	Input,
	Button,
	Popconfirm,
	Form,
	Select,
	Space,
	notification,
} from "antd"
import { decodeUnicode } from "../../../utils/helper"
//import primaryMenu from "../../../settings/primary-menu.json"

const { Option } = Select

const EditableContext = React.createContext(null)

const EditableRow = ({ index, ...props }) => {
	const [form] = Form.useForm()
	return (
		<Form form={form} component={false}>
			<EditableContext.Provider value={form}>
				<tr {...props} />
			</EditableContext.Provider>
		</Form>
	)
}

const EditableCell = ({
	title,
	editable,
	children,
	inputType,
	dataIndex,
	record,
	handleEdit,
	...restProps
}) => {
	const [editing, setEditing] = useState(false)
	const inputRef = useRef(null)
	const form = useContext(EditableContext)
	useEffect(() => {
		if (editing) {
			inputRef.current.focus()
		}
	}, [editing])

	const toggleEdit = () => {
		setEditing(!editing)
		form.setFieldsValue({
			[dataIndex]: record[dataIndex],
		})
	}

	const saveEdit = async () => {
		try {
			const values = await form.validateFields()
			toggleEdit()
			handleEdit({ ...record, ...values })
		} catch (errInfo) {
			console.log("Save edit failed:", errInfo)
		}
	}

	let childNode = children

	if (editable) {
		childNode = editing ? (
			<Form.Item
				style={{
					margin: 0,
				}}
				name={dataIndex}
				rules={[
					{
						required: true,
						message: `${title} is required.`,
					},
				]}
			>
				{inputType === "select" ? (
					<Select ref={inputRef} onChange={saveEdit} onBlur={saveEdit}>
						<Option value="Internal">Internal</Option>
						<Option value="External">External</Option>
					</Select>
				) : (
					<Input ref={inputRef} onPressEnter={saveEdit} onBlur={saveEdit} />
				)}
			</Form.Item>
		) : (
			<div
				className="editable-cell-value-wrap"
				style={{
					paddingRight: 24,
				}}
				role="button"
				tabIndex="0"
				onClick={toggleEdit}
				onKeyDown={toggleEdit}
			>
				{children}
			</div>
		)
	}

	return <td {...restProps}>{childNode}</td>
}

class EditableTableMenus extends React.Component {
	constructor(props) {
		super(props)
		this.columns = [
			{
				title: "Slug",
				dataIndex: "slug",
				editable: true,
			},
			{
				title: "Title",
				dataIndex: "title",
				editable: true,
			},
			{
				title: "Path",
				dataIndex: "path",
				editable: true,
			},
			{
				title: "Link",
				dataIndex: "link",
				editable: true,
				render: (_, record) => {
					return (
						<Select defaultValue={record.link}>
							<Option value="Internal">Internal</Option>
							<Option value="External">External</Option>
						</Select>
					)
				},
			},
			{
				title: "Operation",
				dataIndex: "operation",
				align: "right",
				fixed: "right",
				render: (_, record, rowIndex) => {
					return (
						<Space size="middle">
							<Popconfirm
								title="Sure to add?"
								onConfirm={() => this.handleAddMenu(record, rowIndex)}
							>
								<a href="#add">Add</a>
							</Popconfirm>
							<Popconfirm
								title="Sure to delete?"
								onConfirm={() => this.handleDeleteMenu(record)}
							>
								<a href="#delete">Delete</a>
							</Popconfirm>
						</Space>
					)
				},
			},
		]
		this.state = {
			dataSource: null,
			sha: "",
		}
	}

	handleDeleteMenu = (row) => {
		if (row) {
			const { dataSource } = this.state
			if (row.child === "true") {
				const removeData = dataSource.map((root, i) => {
					const children = root.children
					if (children && children.length > 0) {
						for (var j = 0; j < children.length; j++) {
							if (children[j].key === row.key) {
								children.splice(j, 1)
							}
						}
					}
					return root
				})
				this.setState({
					dataSource: removeData,
				})
			} else {
				this.setState({
					dataSource: dataSource.filter((item) => item.key !== row.key),
				})
			}
		}
	}

	handleAddMenu = (row, index) => {
		const newData = [...this.state.dataSource]
		if (row) {
			const isChild = row.child
			if (isChild === "true") {
				const addData = this.state.dataSource.map((root) => {
					let children = root.children
					if (children && children.length > 0) {
						for (var j = 0; j < children.length; j++) {
							if (children[j].key === row.key) {
								const menuData = {
									key: Date.now(),
									slug: "new-child",
									title: "New child",
									path: "/new-child",
									link: "Internal",
									child: "true",
								}
								children.splice(j, 0, menuData)
								break
							}
						}
					}
					return root
				})
				this.setState({
					dataSource: addData,
				})
			} else {
				this.state.dataSource.map((root) => {
					if (root.key === row.key) {
						const menuData = {
							key: Date.now(),
							slug: "new",
							title: "New",
							path: "/new",
							link: "Internal",
							child: "false",
							children: [
								{
									key: Date.now() + 1,
									slug: "new-child",
									title: "New child",
									path: "/new-child",
									link: "Internal",
									child: "true",
								},
							],
						}
						newData.splice(index, 0, menuData)
					}
					return root
				})
				this.setState({
					dataSource: newData,
				})
			}
		}
	}

	handleEdit = (row) => {
		const newData = [...this.state.dataSource]
		if (row.child === "false") {
			this.state.dataSource.map((root, i) => {
				if (root.key === row.key) {
					newData.splice(i, 1, row)
				}
				return root
			})
			this.setState({
				dataSource: newData,
			})
		} else {
			this.state.dataSource.map((root, i) => {
				const children = root.children
				if (children && children.length > 0) {
					children.map((child, j) => {
						if (child.key === row.key) {
							root.children[j] = row
							newData.splice(i, 1, root)
						}
						return child
					})
				}
				return root
			})
			this.setState({
				dataSource: newData,
			})
		}
	}

	async handleGitGet() {
		try {
			const fetchGitApi = await fetch(
				`https://api.github.com/repos/${process.env.GATSBY_GITHUB_OWNER}/${process.env.GATSBY_GITHUB_REPO}/contents/src/@maad-dev/gatsby-theme-backend/settings/primary-menu.json`,
				{
					method: "GET",
					headers: {
						authorization:
							"token " + process.env.GATSBY_GITHUB_PERSONAL_ACCESS_TOKEN,
						"Content-Type": "application/json; charset=UTF-8",
					},
				}
			)
				.then((response) => {
					return response.json()
				})
				.catch((error) => {
					console.log("Error ", error)
					notification["error"]({
						message: "Error code " + error.status,
						description: error.statusText,
						duration: 0,
					})
				})

			if (fetchGitApi) {
				return fetchGitApi
			} else {
				notification["info"]({
					message: "Empty Fetch",
				})
			}
		} catch (error) {
			console.log("Error handleGitGet ", error)
			notification["error"]({
				message: error.name + " code " + error.status,
				description: error.statusText,
				duration: 0,
			})
		}
		return false
	}
	async handleGitUpdate(contentStr) {
		try {
			const jsonStr = JSON.stringify(contentStr)
			const data = {
				message:
					"[skip ci] Update @maad-dev/gatsby-theme-backend/settings/primary-menu.json from Modern backend.",
				content: btoa(jsonStr),
				sha: this.state.sha,
			}
			const fetchGitApi = await fetch(
				`https://api.github.com/repos/${process.env.GATSBY_GITHUB_OWNER}/${process.env.GATSBY_GITHUB_REPO}/contents/src/@maad-dev/gatsby-theme-backend/settings/primary-menu.json`,
				{
					method: "PUT",
					headers: {
						authorization:
							"token " + process.env.GATSBY_GITHUB_PERSONAL_ACCESS_TOKEN,
						"Content-Type": "application/json; charset=UTF-8",
					},
					body: JSON.stringify(data),
				}
			)
				.then((response) => {
					return response.json()
				})
				.catch((error) => {
					console.log("Error fetchGitApi PUT ", error)
					notification["error"]({
						message: "Error code " + error.status,
						description: error.statusText,
						duration: 0,
					})
				})

			if (fetchGitApi) {
				return true
			} else {
				notification["info"]({
					message: "Empty Fetch",
				})
			}
		} catch (error) {
			console.log("Error handleGitUpdate!", error)
			notification["error"]({
				message: error.name + " code " + error.status,
				description: error.statusText,
				duration: 0,
			})
		}
		return false
	}

	handleSaveMenu = () => {
		const { dataSource } = this.state
		this.handleGitUpdate(dataSource).then((updateFile) => {
			if (updateFile) {
				notification["success"]({
					message: "Success!",
					description: "Updated primary menu successfully.",
				})
			} else {
				let errMsg = "No status"
				if (updateFile === 404) {
					errMsg = "Status: 404 Not Found"
				} else if (updateFile === 409) {
					errMsg = "Status: 409 Conflict"
				} else if (updateFile === 422) {
					errMsg = "Status: 422 Unprocessable Entity"
				}
				notification["error"]({
					message: "Error!",
					description: errMsg,
					duration: 0,
				})
			}
		})
	}

	async componentDidMount() {
		await this.handleGitGet().then((getFile) => {
			if (getFile !== false) {
				const decodeContent = decodeUnicode(getFile.content)
				this.setState({
					dataSource: JSON.parse(decodeContent),
					sha: getFile.sha,
				})
			}
		})
	}

	render() {
		const { dataSource } = this.state
		const components = {
			body: {
				row: EditableRow,
				cell: EditableCell,
			},
		}
		const columns = this.columns.map((col) => {
			if (!col.editable) {
				return col
			}

			return {
				...col,
				onCell: (record) => ({
					record,
					editable: col.editable,
					inputType: col.dataIndex === "link" ? "select" : "text",
					dataIndex: col.dataIndex,
					title: col.title,
					handleEdit: this.handleEdit,
				}),
			}
		})

		return (
			<div className="editableMenusContainer">
				<div className="topWrapper">
					<p>Primary menu</p>
					<Button onClick={this.handleSaveMenu} type="primary">
						Save
					</Button>
				</div>
				<Table
					rowKey="key"
					components={components}
					rowClassName={() => "editable-row"}
					dataSource={dataSource}
					columns={columns}
					pagination={false}
				/>
			</div>
		)
	}
}

export default EditableTableMenus
