import React from "react"
import ThemeContext from "../context/ThemeContext"
import { IntlProvider } from "react-intl"
import LangSwitcher from "./Backend/LangSwitcher"
import { Link } from "gatsby"
import { ConfigProvider } from "antd"
import enUS from "antd/lib/locale/en_US"
import thTH from "antd/lib/locale/th_TH"
import zhCN from "antd/lib/locale/zh_CN"
import enMessages from "../messages/en.json"
import thMessages from "../messages/th.json"
import zhMessages from "../messages/zh.json"
import logo from "../../static/images/logo.svg"
import "antd/dist/antd.less"
import "./Layout.less"

const intlMap = {
	enUS,
	thTH,
	zhCN,
}
const messagesMap = {
	enMessages,
	thMessages,
	zhMessages,
}

const Layout = ({ children }) => (
	<ThemeContext.Consumer>
		{({ currentLang, currentLocale, currentMessage }) => (
			<IntlProvider
				locale={currentLocale}
				messages={messagesMap[currentMessage]}
			>
				<ConfigProvider locale={intlMap[currentLang]}>
					<div className="container">
						<div className="languagesContainer">
							<LangSwitcher />
						</div>
						<div className="content">
							<div className="top">
								<div className="header">
									<Link to="/">
										<img alt="logo" className="logo" src={logo} />
										<span className="title">Modern Backend</span>
									</Link>
								</div>
								<div className="desc">The modern cms for you.</div>
							</div>
							{children}
						</div>
					</div>
				</ConfigProvider>
			</IntlProvider>
		)}
	</ThemeContext.Consumer>
)

export default Layout
